<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:file="http://expath.org/ns/file"
  xmlns:bin="http://expath.org/ns/binary"
  xmlns:my="my-stuff"
  exclude-result-prefixes="#all"
  version="3.0">

  <!-- input parameters from Oxygen's refactoring UI -->
  <xsl:param name="attribute.names" as="xs:string"/>
  <xsl:param name="to.files" as="xs:string"/>

  <!-- convert the attribute name string to a string list -->
  <xsl:variable name="attribute-name-list" as="xs:string*" select="tokenize($attribute.names, '\s+')"/>

  <!-- compute our output directory name -->
  <xsl:variable name="input-doc" select="/"/>
  <xsl:variable name="output-dir" as="xs:anyURI" select="xs:anyURI(base-uri($input-doc) || '-profiling')"/>

  <!-- find the .xpr profiling condition elements to convert -->
  <xsl:variable name="xpr-color-definitions" as="element()*" select="//profilingAttributeStyle[my:get-field(., 'attributeName') = $attribute-name-list]"/>
  <xsl:variable name="xpr-value-definitions" as="element()*" select="//profileConditionInfo[my:get-field(., 'attributeName') = $attribute-name-list]"/>


  <!-- process the input .xpr file -->
  <xsl:mode on-no-match="shallow-copy"/>
  <xsl:template match="$input-doc">

    <!-- create files only if we have what we need -->
    <xsl:choose>
      <xsl:when test="empty($attribute-name-list)">
        <xsl:message>No profiling condition attributes specified for conversion.</xsl:message>
        <xsl:copy-of select="."/>
      </xsl:when>
      <xsl:when test="empty($xpr-color-definitions) and empty($xpr-value-definitions)">
        <xsl:message>No profiling condition value or color definitions found to convert.</xsl:message>
        <xsl:copy-of select="."/>
      </xsl:when>
      <xsl:otherwise>
        <!-- delete any previous output directory -->
        <xsl:if test="file:exists($output-dir)">
          <xsl:sequence select="file:delete($output-dir, true())"/>
        </xsl:if>

        <!-- generate the all-colors.ditaval file -->
        <xsl:result-document href="{$output-dir}/all-colors.ditaval" indent="yes">
          <xsl:processing-instruction name="xml-model">href="urn:oasis:names:tc:dita:xsd:ditaval.rng" schematypens="http://relaxng.org/ns/structure/1.0"</xsl:processing-instruction>
          <val>
            <xsl:perform-sort>
              <xsl:sort select="local-name()" order="ascending"/>
              <xsl:sort select="@att" order="ascending"/>
              <xsl:sort select="@val" order="ascending"/>  <!-- this puts an undefined @val first as a default -->
              <xsl:apply-templates select="*" mode="make-prop"/>
            </xsl:perform-sort>
          </val>
        </xsl:result-document>

        <!-- gather <subjectdef> and <enumerationdef> elements first -->
        <xsl:variable name="subjectdef-stuff" as="element()*">
          <xsl:apply-templates select="*" mode="make-subjectdef"/>
          <xsl:apply-templates select="*" mode="make-enumerationdef"/>
        </xsl:variable>
        <xsl:if test="count($subjectdef-stuff) = 0">
          <xsl:message>No profiling conditions found.</xsl:message>
        </xsl:if>

        <!-- now write the elements to files according to their @result-href attributes -->
        <xsl:for-each-group select="$subjectdef-stuff" group-by="@result-href">
          <xsl:result-document href="{$output-dir}/{current-grouping-key()}" indent="yes">
            <xsl:processing-instruction name="xml-model">href="urn:oasis:names:tc:dita:spec:classification:rng:subjectScheme.rng" schematypens="http://relaxng.org/ns/structure/1.0"</xsl:processing-instruction>
            <subjectScheme>
              <!-- if we're creating the main profiling.ditamap file, include other subjectdef files as needed -->
              <xsl:if test="current-grouping-key() eq 'profiling.ditamap'">
                <xsl:for-each-group select="$subjectdef-stuff/@result-href[not(. eq current-grouping-key())]" group-by=".">
                  <schemeref href="{current-grouping-key()}"/>
                </xsl:for-each-group>
              </xsl:if>

              <!-- write the <subjectdef>/<enumerationdef> elements for this file -->
              <xsl:apply-templates select="current-group()" mode="clean"/>
            </subjectScheme>
          </xsl:result-document>
        </xsl:for-each-group>

        <!-- remove the converted profiling conditions -->
        <xsl:next-match/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- do some post-processing cleanup on output files -->
  <xsl:mode name="clean" on-no-match="shallow-copy"/>
  <xsl:template match="@result-href" mode="clean"/>

  <!-- delete profiling conditions from project file -->
  <xsl:template match="text()[my:is-indenting-whitespace(.)][following-sibling::node()[1][self::
      profilingAttributeStyle[my:get-field(., 'attributeName') = $attribute-name-list]]]"/>
  <xsl:template match="$xpr-color-definitions"/>
  <xsl:template match="text()[my:is-indenting-whitespace(.)][following-sibling::node()[1][self::
      profileConditionInfo[my:get-field(., 'attributeName') = $attribute-name-list]]]"/>
  <xsl:template match="$xpr-value-definitions"/>


  <!-- make a DITAVAL <prop> element from a <profilingAttributeStyle> element -->
  <xsl:mode name="make-prop" on-no-match="shallow-skip"/>
  <xsl:template match="$xpr-color-definitions" mode="make-prop">
    <!-- create a preliminary <prop> element based on the .xpr data -->
    <xsl:variable name="prop" as="element(prop)">
      <prop action="flag">
        <xsl:attribute name="att" select="(my:get-field(., 'attributeGroupName')[not(. = ('', '[ANY]'))], my:get-field(., 'attributeName'))[1]"/>
        <xsl:attribute name="val" select="my:get-field(., 'attributeValue')"/>
        <xsl:attribute name="color" select="my:color-int-to-rgb(my:get-field(., 'foregroundColor'))"/>
        <xsl:attribute name="backcolor" select="my:color-int-to-rgb(my:get-field(., 'backgroundColor'))"/>
        <xsl:variable name="style-keywords" as="xs:string*">
          <xsl:sequence select="('bold')[my:get-field(current(), 'bold') eq 'true']"/>
          <xsl:sequence select="('italic')[my:get-field(current(), 'italic') eq 'true']"/>
          <xsl:sequence select="('underline')[my:get-field(current(), 'textDecoration') eq 'Underline']"/>
          <xsl:sequence select="('overline')[my:get-field(current(), 'textDecoration') eq 'Overline']"/>
          <xsl:sequence select="('double-underline')[my:get-field(current(), 'textDecoration') eq 'Double-underline']"/>
        </xsl:variable>
        <xsl:attribute name="style" select="string-join($style-keywords, ' ')"/>
      </prop>
    </xsl:variable>

    <!-- do some post-processing cleanup on <prop> elements -->
    <xsl:apply-templates select="$prop" mode="clean-prop"/>
  </xsl:template>

  <!-- do a bit of cleanup on <prop> elements -->
  <xsl:mode name="clean-prop" on-no-match="shallow-copy"/>
  <xsl:template match="@val[. = '[ANY]']" mode="clean-prop"/>
  <xsl:template match="(@color|@backcolor|@style)[. = '']" mode="clean-prop"/>
  <xsl:template match="prop[@att eq 'rev']" mode="clean-prop">
    <revprop>
      <xsl:apply-templates select="(@* except @att)|node()" mode="#current"/>
    </revprop>
  </xsl:template>


  <!-- make a <subjectdef> element from a <profileConditionInfo> element -->
  <xsl:mode name="make-subjectdef" on-no-match="shallow-skip"/>
  <xsl:template match="$xpr-value-definitions" mode="make-subjectdef">
    <xsl:variable name="attribute-name" as="xs:string" select="my:get-field(., 'attributeName')"/>
    <xsl:variable name="conditions" as="element()+" select="my:get-field(., 'allowedValues')/*"/>
    <xsl:for-each-group select="$conditions" group-by="(my:get-field(., 'groupAttribute'), '')[1]">
      <xsl:variable name="group-name" as="xs:string?" select="current-grouping-key()[. ne '']"/>
      <xsl:variable name="group-or-attribute-name" as="xs:string" select="($group-name, $attribute-name)[1]"/>
      <subjectdef keys="subjectdefs-{$group-or-attribute-name}">
        <xsl:attribute name="result-href">
          <xsl:choose>
            <xsl:when test="$to.files = 'one'">profiling.ditamap</xsl:when>
            <xsl:when test="$to.files = 'two'">subjectdefs.ditamap</xsl:when>
            <xsl:when test="$to.files = 'many'" expand-text="yes">subjectdefs-{$group-or-attribute-name}.ditamap</xsl:when>
          </xsl:choose>
        </xsl:attribute>
        <topicmeta>
          <navtitle>
            <xsl:choose expand-text="true">
              <xsl:when test="$group-name" >@{$attribute-name} = "{$group-name}(...)" values</xsl:when>
              <xsl:otherwise>@{$attribute-name} values</xsl:otherwise>
            </xsl:choose>
          </navtitle>
        </topicmeta>
        <xsl:for-each select="current-group()">
          <xsl:variable name="value" as="xs:string" select="my:get-field(., 'value')"/>
          <xsl:variable name="value-label" as="xs:string?" select="my:get-field(., 'renderName')[. ne '']"/>
          <subjectdef keys="{if ($group-name) then ($group-name || '(' || $value || ')') else ($value)}">
            <topicmeta>
              <navtitle xsl:expand-text="yes">{($value-label, '(enter description here)')[1]}</navtitle>
            </topicmeta>
          </subjectdef>
        </xsl:for-each>
      </subjectdef>
    </xsl:for-each-group>
  </xsl:template>


  <!-- make an <enumerationdef> element from a <profileConditionInfo> element -->
  <xsl:mode name="make-enumerationdef" on-no-match="shallow-skip"/>
  <xsl:template match="$xpr-value-definitions" mode="make-enumerationdef">
    <xsl:variable name="attribute-name" as="xs:string" select="my:get-field(., 'attributeName')"/>
    <xsl:variable name="conditions" as="element()+" select="my:get-field(., 'allowedValues')/*"/>
    <enumerationdef result-href="profiling.ditamap">
      <attributedef name="{$attribute-name}"/>
      <xsl:for-each-group select="$conditions" group-by="(my:get-field(., 'groupAttribute'), '')[1]">
        <xsl:variable name="group-name" as="xs:string?" select="current-grouping-key()[. ne '']"/>
        <xsl:variable name="group-or-attribute-name" as="xs:string" select="($group-name, $attribute-name)[1]"/>
        <subjectdef keyref="subjectdefs-{$group-or-attribute-name}"/>
      </xsl:for-each-group>
    </enumerationdef>
  </xsl:template>




  <!-- HELPER FUNCTIONS -->

  <!-- return the value of a <field> element in an Oxygen .xpr project file -->
  <xsl:function name="my:get-field">
    <xsl:param name="fields" as="element()?"/>
    <xsl:param name="field-name" as="xs:string"/>
    <xsl:variable name="field-data" as="element()?" select="$fields/field[@name = $field-name]/*"/>

    <xsl:choose>
      <xsl:when test="local-name($field-data) eq 'Boolean'">
        <xsl:value-of select="xs:boolean($field-data)"/>
      </xsl:when>
      <xsl:when test="local-name($field-data) eq 'Integer'">
        <xsl:value-of select="xs:int($field-data)"/>
      </xsl:when>
      <xsl:when test="local-name($field-data) eq 'String'">
        <xsl:value-of select="xs:string($field-data)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="$field-data"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!-- convert a 32-bit signed integer color to a 6-character hex "#RRGGBB" string -->
  <xsl:function name="my:color-int-to-rgb" as="xs:string?">
    <xsl:param name="int" as="xs:integer"/>
    <xsl:variable name="argb" as="xs:string" select="bin:pack-integer($int, 8, 'BE') => xs:hexBinary() => xs:string() => substring(9, 8)"/>

    <!-- return empty sequence if the color is transparent (alpha == "00") -->
    <xsl:if test="not(starts-with($argb, '00'))">
      <xsl:value-of select="'#' || substring($argb, 3, 6)"/>
    </xsl:if>
  </xsl:function>

  <!-- matches element-indenting whitespace -->
  <xsl:function name="my:is-indenting-whitespace" as="xs:boolean">
    <xsl:param name="text" as="text()"/>
    <xsl:value-of select="[matches($text, '^\s*\n\s*$')]"/>
  </xsl:function>

</xsl:stylesheet>
